@@ -1,12 +1,12 @@
 /*
- * Copyright 2005-2008 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright (c) 2005, 2011, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 only, as
- * published by the Free Software Foundation.  Sun designates this
+ * published by the Free Software Foundation.  Oracle designates this
  * particular file as subject to the "Classpath" exception as provided
- * by Sun in the LICENSE file that accompanied this code.
+ * by Oracle in the LICENSE file that accompanied this code.
  *
  * This code is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -18,14 +18,13 @@
  * 2 along with this work; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
- * CA 95054 USA or visit www.sun.com if you need additional information or
- * have any questions.
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
  */
 
 package com.sun.tools.javac.api;
 
-import com.sun.tools.javac.parser.JavacParser;
 import java.io.File;
 import java.io.IOException;
 import java.nio.CharBuffer;
@@ -46,6 +45,7 @@ import com.sun.tools.javac.comp.*;
 import com.sun.tools.javac.file.JavacFileManager;
 import com.sun.tools.javac.main.*;
 import com.sun.tools.javac.model.*;
+import com.sun.tools.javac.parser.JavacParser;
 import com.sun.tools.javac.parser.Parser;
 import com.sun.tools.javac.parser.ParserFactory;
 import com.sun.tools.javac.tree.*;
@@ -55,9 +55,9 @@ import com.sun.tools.javac.util.List;
 import com.sun.tools.javac.main.JavaCompiler;
 
 /**
- * Provides access to functionality specific to the Sun Java Compiler, javac.
+ * Provides access to functionality specific to the JDK Java Compiler, javac.
  *
- * <p><b>This is NOT part of any API supported by Sun Microsystems.
+ * <p><b>This is NOT part of any supported API.
  * If you write code that depends on this, you do so at your own
  * risk.  This code and its internal interfaces are subject to change
  * or deletion without notice.</b></p>
@@ -66,7 +66,7 @@ import com.sun.tools.javac.main.JavaCompiler;
  * @author Jonathan Gibbons
  */
 public class JavacTaskImpl extends JavacTask {
-    private JavacTool tool;
+    private ClientCodeWrapper ccw;
     private Main compilerMain;
     private JavaCompiler compiler;
     private Locale locale;
@@ -81,12 +81,11 @@ public class JavacTaskImpl extends JavacTask {
     
     private Integer result = null;
 
-    JavacTaskImpl(JavacTool tool,
-                Main compilerMain,
+    JavacTaskImpl(Main compilerMain,
                 String[] args,
                 Context context,
                 List<JavaFileObject> fileObjects) {
-        this.tool = tool;
+        this.ccw = ClientCodeWrapper.instance(context);
         this.compilerMain = compilerMain;
         this.args = args;
         this.context = context;
@@ -95,20 +94,15 @@ public class JavacTaskImpl extends JavacTask {
         // null checks
         compilerMain.getClass();
         args.getClass();
-        context.getClass();
         fileObjects.getClass();
-
-        // force the use of the scanner that captures Javadoc comments
-        com.sun.tools.javac.parser.DocCommentScanner.Factory.preRegister(context);
     }
 
-    JavacTaskImpl(JavacTool tool,
-                Main compilerMain,
+    JavacTaskImpl(Main compilerMain,
                 Iterable<String> flags,
                 Context context,
                 Iterable<String> classes,
                 Iterable<? extends JavaFileObject> fileObjects) {
-        this(tool, compilerMain, toArray(flags, classes), context, toList(fileObjects));
+        this(compilerMain, toArray(flags, classes), context, toList(fileObjects));
     }
 
     JavacTaskImpl(Context context) {
@@ -138,19 +132,11 @@ public class JavacTaskImpl extends JavacTask {
 
     public Boolean call() {
         if (!used.getAndSet(true)) {
-            beginContext();
+            initContext();
             notYetEntered = new HashMap<JavaFileObject, JCCompilationUnit>();
-            try {
-                compilerMain.setFatalErrors(true);
-                result = compilerMain.compile(args, context, fileObjects, processors);
-            } finally {
-                endContext();
-            }
-            compilerMain = null;
-            args = null;
-            context = null;
-            fileObjects = null;
-            notYetEntered = null;
+            compilerMain.setAPIMode(true);
+            result = compilerMain.compile(args, context, fileObjects, processors);
+            cleanup();
             return result == 0;
         } else {
             throw new IllegalStateException("multiple calls to method 'call'");
@@ -176,8 +162,11 @@ public class JavacTaskImpl extends JavacTask {
     }
 
     private void prepareCompiler() throws IOException {
-        if (!used.getAndSet(true)) {
-            beginContext();
+        if (used.getAndSet(true)) {
+            if (compiler == null)
+                throw new IllegalStateException();
+        } else {
+            initContext();
             compilerMain.setOptions(Options.instance(context));
             compilerMain.filenames = new ListBuffer<File>();
             List<File> filenames = compilerMain.processArgs(CommandLine.parse(args));
@@ -197,41 +186,25 @@ public class JavacTaskImpl extends JavacTask {
         }
     }
 
-    private void beginContext() {
+    private void initContext() {
         context.put(JavacTaskImpl.class, this);
         if (context.get(TaskListener.class) != null)
             context.put(TaskListener.class, (TaskListener)null);
         if (taskListener != null)
-            context.put(TaskListener.class, wrap(taskListener));
-        tool.beginContext(context);
+            context.put(TaskListener.class, ccw.wrap(taskListener));
         //initialize compiler's default locale
-        JavacMessages.instance(context).setCurrentLocale(locale);
-    }
-    // where
-    private TaskListener wrap(final TaskListener tl) {
-        tl.getClass(); // null check
-        return new TaskListener() {
-            public void started(TaskEvent e) {
-                try {
-                    tl.started(e);
-                } catch (Throwable t) {
-                    throw new ClientCodeException(t);
-                }
-            }
-
-            public void finished(TaskEvent e) {
-                try {
-                    tl.finished(e);
-                } catch (Throwable t) {
-                    throw new ClientCodeException(t);
-                }
-            }
-
-        };
+        context.put(Locale.class, locale);
     }
 
-    private void endContext() {
-        tool.endContext();
+    void cleanup() {
+        if (compiler != null)
+            compiler.close();
+        compiler = null;
+        compilerMain = null;
+        args = null;
+        context = null;
+        fileObjects = null;
+        notYetEntered = null;
     }
 
     /**
@@ -382,9 +355,14 @@ public class JavacTaskImpl extends JavacTask {
         try {
             List<JCCompilationUnit> units = compiler.enterTrees(roots.toList());
 
-            if (!compiler.skipAnnotationProcessing)
-                compiler = compiler.processAnnotations(units);
-
+            if (!compiler.skipAnnotationProcessing) {
+                compiler.skipAnnotationProcessing = true;
+                try {
+                    compiler.processAnnotations(units);
+                } finally {
+                    compiler.skipAnnotationProcessing = false;
+                }
+            }
             compiler.flushTempDiags();
 
             ListBuffer<TypeElement> elements = new ListBuffer<TypeElement>();
@@ -395,9 +373,13 @@ public class JavacTaskImpl extends JavacTask {
                     if (unit.packge.package_info != null)
                         elements.append(unit.packge.package_info);
                 } else {
-                    for (JCTree node : unit.defs)
-                        if (node.getTag() == JCTree.CLASSDEF && ((JCTree.JCClassDecl) node).sym != null)
-                            elements.append(((JCTree.JCClassDecl) node).sym);
+                    for (JCTree node : unit.defs) {
+                        if (node.getTag() == JCTree.CLASSDEF) {
+                            JCClassDecl cdef = (JCClassDecl) node;
+                            if (cdef.sym != null) // maybe null if errors in anno processing
+                                elements.append(cdef.sym);
+                        }
+                    }
                 }
             }
             return elements.toList();
@@ -548,11 +530,11 @@ public class JavacTaskImpl extends JavacTask {
                 compiler.reportDeferredDiagnostics();
                 compiler.log.flush();
                 compiler.repair.flush();
-                endContext();
             }
         }
         finally {
-            compiler.log.flush();
+            if (compiler != null)
+                compiler.log.flush();
         }
         return results;
     }
@@ -582,7 +564,7 @@ public class JavacTaskImpl extends JavacTask {
         if (this.genList != null && !this.genList.isEmpty()) {
             this.genList.clear();
         }
-        endContext();
+        cleanup();
     }
 
     public TypeMirror getTypeMirror(Iterable<? extends Tree> path) {
@@ -639,7 +621,7 @@ public class JavacTaskImpl extends JavacTask {
     }
 
     /**
-     * For internal use by Sun Microsystems only.  This method will be
+     * For internal use only.  This method will be
      * removed without warning.
      */
     public Context getContext() {
@@ -647,7 +629,7 @@ public class JavacTaskImpl extends JavacTask {
     }
 
     /**
-     * For internal use by Sun Microsystems only.  This method will be
+     * For internal use only.  This method will be
      * removed without warning.
      */
     public void updateContext(Context newContext) {
@@ -655,7 +637,7 @@ public class JavacTaskImpl extends JavacTask {
     }
 
     /**
-     * For internal use by Sun Microsystems only.  This method will be
+     * For internal use only.  This method will be
      * removed without warning.
      */
     public Type parseType(String expr, TypeElement scope) {
@@ -678,99 +660,106 @@ public class JavacTaskImpl extends JavacTask {
         }
     }
 
-    public JCStatement parseStatement(CharSequence stmt, SourcePositions[] pos) {
-        if (stmt == null || (pos != null && pos.length != 1))
+    public Tree parseType(String expr) {
+        if (expr == null || expr.equals(""))
             throw new IllegalArgumentException();
         compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
         boolean old = compiler.log.suppressErrorsAndWarnings;
         compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
-        try {
-            CharBuffer buf = CharBuffer.wrap((stmt+"\u0000").toCharArray(), 0, stmt.length());
-            Parser parser = parserFactory.newParser(buf, false, true, false, true);
-            if (parser instanceof JavacParser) {
-                if (pos != null)
-                    pos[0] = new ParserSourcePositions((JavacParser)parser);
-                return parser.parseStatement();
-            }
-            return null;
-        } finally {
-            compiler.log.suppressErrorsAndWarnings = old;
-            compiler.log.useSource(prev);
-        }
-    }
-
-    public JCExpression parseExpression(CharSequence expr, SourcePositions[] pos) {
-        if (expr == null || (pos != null && pos.length != 1))
-            throw new IllegalArgumentException();
-            compiler = JavaCompiler.instance(context);
-        JavaFileObject prev = compiler.log.useSource(null);
-        boolean old = compiler.log.suppressErrorsAndWarnings;
-        compiler.log.suppressErrorsAndWarnings = true;
-        ParserFactory parserFactory = ParserFactory.instance(context);
+        Attr attr = Attr.instance(context);
         try {
             CharBuffer buf = CharBuffer.wrap((expr+"\u0000").toCharArray(), 0, expr.length());
-            Parser parser = parserFactory.newParser(buf, false, true, false, true);
-            if (parser instanceof JavacParser) {
-                if (pos != null)
-                    pos[0] = new ParserSourcePositions((JavacParser)parser);
-                return parser.parseExpression();
-            }
-            return null;
+            Parser parser = parserFactory.newParser(buf, false, false, false, true);
+            return parser.parseType();
         } finally {
             compiler.log.suppressErrorsAndWarnings = old;
             compiler.log.useSource(prev);
         }
     }
 
+    public JCStatement parseStatement(CharSequence stmt, SourcePositions[] pos, DiagnosticListener<? super JavaFileObject> errors) {
+        return (JCStatement) doParse(ParseKind.STATEMENT, stmt, pos, errors);
+    }
+
+    private JCTree doParse(ParseKind kind, CharSequence source, SourcePositions[] pos, DiagnosticListener<? super JavaFileObject> errors) {
+        if (source == null || (pos != null && pos.length != 1))
+            throw new IllegalArgumentException();
+        compiler = JavaCompiler.instance(context);
+        JavaFileObject prev = compiler.log.useSource(null);
+        boolean old = compiler.log.suppressErrorsAndWarnings;
+        DiagnosticListener<? super JavaFileObject> oldListener = compiler.log.getDiagnosticListener();
+        int oldErrors = compiler.log.nerrors;
+        int oldWarnings = compiler.log.nwarnings;
+        compiler.log.suppressErrorsAndWarnings = false;
+        compiler.log.setDiagnosticListener(errors);
+        ParserFactory parserFactory = ParserFactory.instance(context);
+        try {
+            CharBuffer buf = CharBuffer.wrap((source+"\u0000").toCharArray(), 0, source.length());
+            Parser parser = parserFactory.newParser(buf, false, true, false, true);
+            if (parser instanceof JavacParser) {
+                if (pos != null)
+                    pos[0] = new ParserSourcePositions((JavacParser)parser);
+                switch (kind) {
+                    case STATEMENT: return parser.parseStatement();
+                    case EXPRESSION: return parser.parseExpression();
+                    case VARIABLE_INIT: return ((JavacParser)parser).variableInitializer();
+                    case STATIC_BLOCK:
+                        List<JCTree> trees = ((JavacParser)parser).classOrInterfaceBodyDeclaration(null, false);
+                        return trees.head != null && trees.head.getTag() == JCTree.BLOCK ? (JCBlock) trees.head : null;
+                    default: throw new UnsupportedOperationException(kind.name());
+                }
+                
+            }
+            return null;
+        } finally {
+            compiler.log.suppressErrorsAndWarnings = old;
+            compiler.log.setDiagnosticListener(oldListener);
+            compiler.log.useSource(prev);
+            compiler.log.nerrors = oldErrors;
+            compiler.log.nwarnings = oldWarnings;
+        }
+    }
+
+    public JCExpression parseExpression(CharSequence expr, SourcePositions[] pos, DiagnosticListener<? super JavaFileObject> errors) {
+        return (JCExpression) doParse(ParseKind.EXPRESSION, expr, pos, errors);
+    }
+
+    public JCExpression parseVariableInitializer(CharSequence init, SourcePositions[] pos, DiagnosticListener<? super JavaFileObject> errors) {
+        return (JCExpression) doParse(ParseKind.VARIABLE_INIT, init, pos, errors);
+    }
+
+    public JCBlock parseStaticBlock(CharSequence block, SourcePositions[] pos, DiagnosticListener<? super JavaFileObject> errors) {
+        return (JCBlock) doParse(ParseKind.STATIC_BLOCK, block, pos, errors);
+    }
+
+    private enum ParseKind {STATEMENT, EXPRESSION, VARIABLE_INIT, STATIC_BLOCK;}
+
+    @Deprecated
+    public JCStatement parseStatement(CharSequence stmt, SourcePositions[] pos) {
+        return parseStatement(stmt, pos, new DiscardDiagnosticListener());
+    }
+
+    @Deprecated
+    public JCExpression parseExpression(CharSequence expr, SourcePositions[] pos) {
+        return parseExpression(expr, pos, new DiscardDiagnosticListener());
+    }
+
+    @Deprecated
     public JCExpression parseVariableInitializer(CharSequence init, SourcePositions[] pos) {
-        if (init == null || (pos != null && pos.length != 1))
-            throw new IllegalArgumentException();
-            compiler = JavaCompiler.instance(context);
-        JavaFileObject prev = compiler.log.useSource(null);
-        boolean old = compiler.log.suppressErrorsAndWarnings;
-        compiler.log.suppressErrorsAndWarnings = true;
-        ParserFactory parserFactory = ParserFactory.instance(context);
-        try {
-            CharBuffer buf = CharBuffer.wrap((init+"\u0000").toCharArray(), 0, init.length());
-            Parser parser = parserFactory.newParser(buf, false, true, false, true);
-            if (parser instanceof JavacParser) {
-                if (pos != null)
-                    pos[0] = new ParserSourcePositions((JavacParser)parser);
-                return ((JavacParser)parser).variableInitializer();
-            }
-            return null;
-        } finally {
-            compiler.log.suppressErrorsAndWarnings = old;
-            compiler.log.useSource(prev);
-        }
+        return parseVariableInitializer(init, pos, new DiscardDiagnosticListener());
     }
 
+    @Deprecated
     public JCBlock parseStaticBlock(CharSequence block, SourcePositions[] pos) {
-        if (block == null || (pos != null && pos.length != 1))
-            throw new IllegalArgumentException();
-            compiler = JavaCompiler.instance(context);
-        JavaFileObject prev = compiler.log.useSource(null);
-        boolean old = compiler.log.suppressErrorsAndWarnings;
-        compiler.log.suppressErrorsAndWarnings = true;
-        ParserFactory parserFactory = ParserFactory.instance(context);
-        try {
-            CharBuffer buf = CharBuffer.wrap((block+"\u0000").toCharArray(), 0, block.length());
-            Parser parser = parserFactory.newParser(buf, false, true, false, true);
-            if (parser instanceof JavacParser) {
-                if (pos != null)
-                    pos[0] = new ParserSourcePositions((JavacParser)parser);
-                List<JCTree> trees = ((JavacParser)parser).classOrInterfaceBodyDeclaration(null, false);
-                return trees.head != null && trees.head.getTag() == JCTree.BLOCK ? (JCBlock) trees.head : null;
-            }
-            return null;
-        } finally {
-            compiler.log.suppressErrorsAndWarnings = old;
-            compiler.log.useSource(prev);
-        }
+        return parseStaticBlock(block, pos, new DiscardDiagnosticListener());
     }
 
+    private static final class DiscardDiagnosticListener implements DiagnosticListener<JavaFileObject> {
+        public void report(Diagnostic<? extends JavaFileObject> diagnostic) {}
+    }
+    
     public Type attributeTree(JCTree tree, Env<AttrContext>env) {
         Log log = Log.instance(context);
         Attr attr = Attr.instance(context);
@@ -883,7 +872,7 @@ public class JavacTaskImpl extends JavacTask {
         StringBuilder res = new StringBuilder ();
         if (compiler != null && compiler.todo != null) {
             for (Env<AttrContext> env : compiler.todo) {
-                res.append(((JCClassDecl)env.tree).sym.toString()  + " from: " +env.toplevel.sourcefile.toUri());
+                res.append(((JCClassDecl)env.tree).sym.toString()).append(" from: ").append(env.toplevel.sourcefile.toUri());
             }
         }
         return res.toString();
